"""
@file    __init__.py
@author  Daniel Krajzewicz
@date    2012-12-04
@version $Id: __init__.py 14732 2013-09-20 20:11:11Z behrisch $

This file includes all shape structures.

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

from . import phem, omnet, ns2, shawn, gpsdat
