
/* --------------------------------------------------

                   GENERATED FILE

               MODIFY AT YOUR OWN RISK

   --------------------------------------------------

This file contains copyrighted information and is used
under terms described in license/README and
license/VeraLicense. Please read the license files
before using this file.

   -------------------------------------------------- */

#ifndef _pfPSansBold16_
#define _pfPSansBold16_

#ifdef _MSC_VER
#include <windows_config.h>
#else
#include <config.h>
#endif

#include "polyfonttypes.h"

#ifdef __cplusplus
extern "C"
{
#endif

    extern pffont pfPSansBold16;

#ifdef __cplusplus
}
#endif

#endif /* _pfPSansBold16_ */
