package enums;

/**
 * Enumeration used to define the current editing mode based on the currently selected layer.
 */
public enum EditingMode {
	Map, GenericTraffic, UserDefinedTraffic, ActivityBasedTraffic, RandomTraffic
}
